package com.usky.dxtop.framework.web.exception;

import com.usky.dxtop.common.constant.HttpStatus;
import com.usky.dxtop.common.core.domain.ApiResult;
import com.usky.dxtop.common.exception.BaseException;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.exception.DemoModeException;
import com.usky.dxtop.common.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.AccountExpiredException;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.validation.BindException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.NoHandlerFoundException;

/**
 * 全局异常处理器
 *
 * @author yq
 */
@RestControllerAdvice
public class GlobalExceptionHandler
{
    private static final Logger log = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    /**
     * 基础异常
     */
    @ExceptionHandler(BaseException.class)
    public ApiResult baseException(BaseException e)
    {
        return ApiResult.error(String.valueOf(HttpStatus.ERROR), e.getMessage());
    }

    /**
     * 业务异常
     */
    @ExceptionHandler(CustomException.class)
    public ApiResult businessException(CustomException e)
    {
        if (StringUtils.isNull(e.getCode()))
        {
            return ApiResult.error(String.valueOf(HttpStatus.ERROR),e.getMessage());
        }
        return ApiResult.error(String.valueOf(e.getCode()), e.getMessage());
    }

    @ExceptionHandler(NoHandlerFoundException.class)
    public ApiResult handlerNoFoundException(Exception e)
    {
        log.error(e.getMessage(), e);
        return ApiResult.error(String.valueOf(HttpStatus.NOT_FOUND), "路径不存在，请检查路径是否正确");
    }

    @ExceptionHandler(AccessDeniedException.class)
    public ApiResult handleAuthorizationException(AccessDeniedException e)
    {
        log.error(e.getMessage());
        return ApiResult.error(String.valueOf(HttpStatus.FORBIDDEN), "没有权限，请联系管理员授权");
    }

    @ExceptionHandler(AccountExpiredException.class)
    public ApiResult handleAccountExpiredException(AccountExpiredException e)
    {
        log.error(e.getMessage(), e);
        return ApiResult.error(String.valueOf(HttpStatus.ERROR),e.getMessage());
    }

    @ExceptionHandler(UsernameNotFoundException.class)
    public ApiResult handleUsernameNotFoundException(UsernameNotFoundException e)
    {
        log.error(e.getMessage(), e);
        return ApiResult.error(String.valueOf(HttpStatus.ERROR),e.getMessage());
    }

    @ExceptionHandler(Exception.class)
    public ApiResult handleException(Exception e)
    {
        log.error(e.getMessage(), e);
        return ApiResult.error(String.valueOf(HttpStatus.ERROR),e.getMessage());
    }

    /**
     * 自定义验证异常
     */
    @ExceptionHandler(BindException.class)
    public ApiResult validatedBindException(BindException e)
    {
        log.error(e.getMessage(), e);
        String message = e.getAllErrors().get(0).getDefaultMessage();
        return ApiResult.error(String.valueOf(HttpStatus.ERROR),message);
    }

    /**
     * 自定义验证异常
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public Object validExceptionHandler(MethodArgumentNotValidException e)
    {
        log.error(e.getMessage(), e);
        String message = e.getBindingResult().getFieldError().getDefaultMessage();
        return ApiResult.error(String.valueOf(HttpStatus.ERROR),message);
    }

    /**
     * 演示模式异常
     */
    @ExceptionHandler(DemoModeException.class)
    public ApiResult demoModeException(DemoModeException e)
    {
        return ApiResult.error(String.valueOf(HttpStatus.ERROR),"演示模式，不允许操作");
    }
}
